"""
.. seealso::
    Support for FDS is provided through a thin wrapper around the `fdsreader <https://github.com/FireDynamics/fdsreader>`_ package developed by `Forschungszentrum Jülich <https://www.fz-juelich.de>`_.
    The full API documentation is available `here <https://firedynamics.github.io/fdsreader/>`_.
    
    Special thanks to the developers for their hard work they've put in to that package.
    
"""

from theng.models.fds.fds import FDSModel